#include "cora_fit.h"

#include <stdlib.h>
#include <stdio.h>

#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <sys/types.h>

double max(Vector *x)
{
  double max;
  int i, n;

  n = x->len;
  
  if (n == 0)
    {
      fprintf (stderr, 
	       "* Warning *: max(): Zero length vector.\n");
      return (0.0);
    }
  max = x->vec[0];

  for(i=1; i < n; ++i)
    if (x->vec[i] > max) 
      max = x->vec[i];
  
  return(max);
}

/*  ------------------------ */

double min(Vector *x)
{
  double min;
  int i,n;

  n = x->len;
  
  if (n == 0)
    {
      fprintf (stderr, 
	       "* Warning *: min(): Zero length vector.\n");
      return (0.0);
    }


  min = x->vec[0];

  for(i = 1; i < n; ++i)
    if (x->vec[i] < min) 
      min=x->vec[i];
  
  return(min);
}

/* ------------------------ */

double mod(double a, double b)
{
  double mod;

  if (b!=0) 
    mod=a-(int)(a/b)*b;
  else 
    return(-1);

  return(mod);
}

/* ------------------------ */

int comp_func(const void *a, const void *b)
{
  double *x, *y;
  x = (double *) a;
  y = (double *) b;

  if (*x < *y)
      return -1;
  else if (*x > *y)
      return  1;
  else
      return 0;
} 

/* ------------------------ */

double median(Vector *list)

{
  double median, mm = 2.0;
  int num;
  
  num=list->len;

  if (num == 0)
    {
      fprintf (stderr, 
	       "* Warning *: median(): Zero length vector.\n");
      return (0.0);
    }

  
  qsort(&(list->vec[0]), num, sizeof(double), comp_func);
  
  /* Calculate median */
  if (mod((double)num, mm) != 0)
    median = list->vec[num/2 + 1];
  else
    median = (list->vec[num/2]+list->vec[num/2 + 1])/2.0;
  
  return(median);
}

/* ------------------------ */

double total(Vector *x)
{
  double total;
  int i,n;

  n=x->len;

  total=0.0;

  for(i=0; i<n; ++i) 
    total = total + x->vec[i];

  return(total);
}

/* ------------------------ */

int test_confdir ()
{
  /*  if configuration directory $HOME/.cora exists and creates it if not
   */
  char path[1024];
  struct stat buf;
  int mask;
  int error;

  sprintf(path, "%s/.cora", get_home_dir()); 

  if (0 != (stat (path, &buf)))
    {
      error = errno;
      if (error == ENOENT)
        {
          mask = umask (0);
          if (-1 == mkdir (path, S_IRWXU))
            {
              error = errno;
              fprintf(stderr,
                    "* Warning *: Error creating directory %s\n", path);
              return (-1);
            } 
          umask (mask);
          return (0);
        }
      else
        {
          fprintf(stderr,
              "* Warning *: Error accessing %s\n", path); 
          return (-1);
        }
    }
  else
    return 0;
}

/* ------------------------ */

char * get_home_dir()
{
struct passwd * pwf_entry;
char *testchar;

  testchar = getenv("HOME");
  if (testchar != NULL)
    return testchar;
  else
   {
    pwf_entry = getpwuid(getuid());
    if (pwf_entry != NULL && pwf_entry->pw_dir != NULL)
        return pwf_entry->pw_dir;
   }
  return NULL;
}

/* ------------------------ */

void print_vec(Vector *amp)
{
 int i,n;

 n=amp->len;

 fprintf(stderr,"Amp= ");
 for(i=0;i<n;++i) fprintf(stderr,"%f ",amp->vec[i]);
 fprintf(stderr,"\n");

  return;
}

